module HDF5.Base (Hid(..),
                  Did(..),
                  Eid(..),
                  Lid(..),
                  Pid(..),
                  PCid(..),
                  Sid(..),
                  Tid(..),
                  Herr(..),
                  Hsize(..),
                  dimensions)
where

import Foreign
import Foreign.C
import Data.Ix
import Control.Monad (liftM)

newtype Did  = Did  CInt
newtype Eid  = Eid  CInt deriving (Eq,Show)
newtype Lid  = Lid  CInt
newtype Pid  = Pid  CInt
newtype PCid = PCid CInt
newtype Sid  = Sid  CInt
newtype Tid  = Tid  CInt

newtype Herr = Herr CInt

class Hid a where
  isFailure :: a -> Bool

instance Hid Did where
  isFailure (Did x) = x < 0

instance Hid Eid where
  isFailure (Eid x) = x < 0

instance Hid Lid where
  isFailure (Lid x) = x < 0

instance Hid Pid where
  isFailure (Pid x) = x < 0

instance Hid Sid where
  isFailure (Sid x) = x < 0

instance Hid Tid where
  isFailure (Tid x) = x < 0

-- First index varies fastest, giving column-major (Fortran) storage
newtype Hsize = Hsize [CULLong]
              deriving (Eq,Ord,Show)

instance Ix Hsize where
  range (Hsize []    ,Hsize [])     = [Hsize []] -- rank 0 (scalar)
  range (Hsize (l:ls),Hsize (u:us)) = [Hsize (head:tail)
                                      | Hsize tail <- range (Hsize ls,Hsize us)
                                      ,       head <- [l..u]]
  inRange (Hsize ls,Hsize us) (Hsize as) = and $ zipWith3 (\l u a -> l<=a && a<=u) ls us as
  index b@(Hsize ll,Hsize uu) elem@(Hsize aa) = fromIntegral $ index' ll uu aa
    where index' []     []     []     = 0
          index' (l:ls) (u:us) (a:as)
            | l<=a && a<=u = a-l + (u-l+1) * index' ls us as
            | otherwise    = error $ unwords ["Error in array index:",show b,show elem]

dimensions :: (Hsize,Hsize) -> [CULLong]
dimensions (Hsize ls,Hsize us) = zipWith (\l u -> u-l+1) ls us
-- FIXME maybe move error checking here from index and use this in index
