{-# LANGUAGE ForeignFunctionInterface #-}
module HDF5.File (OpenFlag(..),
                  CreateFlag(..),
                  create,
                  open,
                  close,
                  withFile,
                  withNewFile)
where

import Foreign
import Foreign.C
import Control.Exception (bracket)

import HDF5.Base
import qualified HDF5.Error as H5E
import qualified HDF5.Property as H5P

newtype Flags = Flags CUInt

acc_rdonly = Flags 0x0000       -- absence of rdwr => rd-only
acc_rdwr   = Flags 0x0001       -- open for read and write
acc_trunc  = Flags 0x0002       -- overwrite existing files
acc_excl   = Flags 0x0004       -- fail if file already exists
acc_debug  = Flags 0x0008       -- print debug info
acc_creat  = Flags 0x0010       -- create non-existing files

data   OpenFlag = RDONLY | RDWR
data CreateFlag =  TRUNC | EXCL

foreign import ccall unsafe "hdf5.h H5Fcreate"
  create_ :: CString -> Flags -> Pid -> Pid -> IO Lid
--          filename   flags create_plist access_plist

create :: FilePath -> CreateFlag -> IO Lid
create filename flag =
  H5E.throwIfErr ("creating "++filename) $
  withCString filename (\fn -> create_ fn hflags H5P._default H5P._default)
    where hflags = case flag of
                     TRUNC -> acc_trunc
                     EXCL  -> acc_excl

foreign import ccall unsafe "hdf5.h H5Fopen"
  open_ :: CString -> Flags -> Pid -> IO Lid
--         filename   flags  access_plist

open :: FilePath -> OpenFlag -> IO Lid
open filename flag =
  H5E.throwIfErr ("opening "++filename) $
  withCString filename (\fn -> open_ fn hflags H5P._default)
    where hflags = case flag of
                     RDONLY -> acc_rdonly
                     RDWR   -> acc_rdwr

foreign import ccall unsafe "hdf5.h H5Fclose"
--        file_id
  close_ :: Lid -> IO Herr

close :: Lid -> IO ()
close =
  H5E.throwIfErr_ "closing file" . close_

withFile :: FilePath -> OpenFlag -> (Lid -> IO a) -> IO a
withFile name flag = bracket (open name flag) close

withNewFile :: FilePath -> CreateFlag -> (Lid -> IO a) -> IO a
withNewFile name flag = bracket (create name flag) close
