module HDF5.Platform (Error(..))
where

import Foreign
import Foreign.C
import Control.Applicative ((<$>),(<*>))
import Control.Monad (liftM)
import Data.Maybe (fromMaybe)

import HDF5.Base

#include <hdf5.h>

data Error = Error { cls_id    :: Eid,
                     maj_num   :: Eid,
                     min_num   :: Eid,
                     line      :: CUInt,
                     func_name :: String,
                     file_name :: String,
                     desc      :: String }
           deriving Show

defaultPeek :: b -> (Ptr a -> IO b) -> Ptr a -> IO b
defaultPeek def peek ptr =
  fromMaybe def `liftM` maybePeek peek ptr

instance Storable Error where
  sizeOf _    = #size H5E_error2_t
  alignment _ = alignment (undefined :: CInt)
  peek p      = Error
                <$> Eid `liftM` (#peek H5E_error2_t, cls_id ) p
                <*> Eid `liftM` (#peek H5E_error2_t, maj_num) p
                <*> Eid `liftM` (#peek H5E_error2_t, min_num) p
                <*> (#peek H5E_error2_t, line) p
                <*> ((#peek H5E_error2_t, func_name) p >>= defaultPeek "(null)" peekCString)
                <*> ((#peek H5E_error2_t, file_name) p >>= defaultPeek "(null)" peekCString)
                <*> ((#peek H5E_error2_t, desc     ) p >>= defaultPeek "(null)" peekCString)
  poke p (Error (Eid cls_id) (Eid maj_num) (Eid min_num) line func_name file_name desc) = do
    (#poke H5E_error2_t, cls_id ) p cls_id
    (#poke H5E_error2_t, maj_num) p maj_num
    (#poke H5E_error2_t, min_num) p min_num
    (#poke H5E_error2_t, line) p line
    (#poke H5E_error2_t, func_name) p nullPtr -- FIXME not implemented
    (#poke H5E_error2_t, file_name) p nullPtr -- FIXME not implemented
    (#poke H5E_error2_t, desc     ) p nullPtr -- FIXME not implemented
