{-# LANGUAGE ForeignFunctionInterface #-}
module HDF5.Property (_default,
                      create,
                      close,
                      set_chunk,
                      dataset_create)
where

import Foreign
import Foreign.C

import HDF5.Base
import qualified HDF5.Error as H5E

_default = Pid 0

foreign import ccall unsafe "hdf5.h H5Pcreate"
  create_ :: PCid -> IO Pid

create :: PCid -> IO Pid
create classP =
  H5E.throwIfErr "creating property list" $ create_ classP

foreign import ccall unsafe "hdf5.h H5Pclose"
  close_ :: Pid -> IO Herr

close :: Pid -> IO ()
close =
  H5E.throwIfErr_ "closing property list" . close_

foreign import ccall unsafe "hdf5.h H5Pset_chunk"
  set_chunk_ :: Pid -> CInt -> Ptr CULLong -> IO Herr

set_chunk :: Pid -> Hsize -> IO ()
set_chunk plist (Hsize dims) =
  withArrayLen (reverse dims) $ \dimsLen dimsP ->
  H5E.throwIfErr_ "setting chunk size" $ set_chunk_ plist (fromIntegral dimsLen) dimsP

peekPCid = PCid . unsafePerformIO . peek

root             = peekPCid root_g
object_create    = peekPCid object_create_g
file_create      = peekPCid file_create_g
file_access      = peekPCid file_access_g
dataset_create   = peekPCid dataset_create_g
dataset_access   = peekPCid dataset_access_g
dataset_xfer     = peekPCid dataset_xfer_g
file_mount       = peekPCid file_mount_g
group_create     = peekPCid group_create_g
group_access     = peekPCid group_access_g
datatype_create  = peekPCid datatype_create_g
datatype_access  = peekPCid datatype_access_g
string_create    = peekPCid string_create_g
attribute_create = peekPCid attribute_create_g
object_copy      = peekPCid object_copy_g
link_create      = peekPCid link_create_g
link_access      = peekPCid link_access_g

-- Raw address imports for the above
foreign import ccall unsafe "& H5P_CLS_ROOT_g"             root_g             :: Ptr CInt
foreign import ccall unsafe "& H5P_CLS_OBJECT_CREATE_g"    object_create_g    :: Ptr CInt
foreign import ccall unsafe "& H5P_CLS_FILE_CREATE_g"      file_create_g      :: Ptr CInt
foreign import ccall unsafe "& H5P_CLS_FILE_ACCESS_g"      file_access_g      :: Ptr CInt
foreign import ccall unsafe "& H5P_CLS_DATASET_CREATE_g"   dataset_create_g   :: Ptr CInt
foreign import ccall unsafe "& H5P_CLS_DATASET_ACCESS_g"   dataset_access_g   :: Ptr CInt
foreign import ccall unsafe "& H5P_CLS_DATASET_XFER_g"     dataset_xfer_g     :: Ptr CInt
foreign import ccall unsafe "& H5P_CLS_FILE_MOUNT_g"       file_mount_g       :: Ptr CInt
foreign import ccall unsafe "& H5P_CLS_GROUP_CREATE_g"     group_create_g     :: Ptr CInt
foreign import ccall unsafe "& H5P_CLS_GROUP_ACCESS_g"     group_access_g     :: Ptr CInt
foreign import ccall unsafe "& H5P_CLS_DATATYPE_CREATE_g"  datatype_create_g  :: Ptr CInt
foreign import ccall unsafe "& H5P_CLS_DATATYPE_ACCESS_g"  datatype_access_g  :: Ptr CInt
foreign import ccall unsafe "& H5P_CLS_STRING_CREATE_g"    string_create_g    :: Ptr CInt
foreign import ccall unsafe "& H5P_CLS_ATTRIBUTE_CREATE_g" attribute_create_g :: Ptr CInt
foreign import ccall unsafe "& H5P_CLS_OBJECT_COPY_g"      object_copy_g      :: Ptr CInt
foreign import ccall unsafe "& H5P_CLS_LINK_CREATE_g"      link_create_g      :: Ptr CInt
foreign import ccall unsafe "& H5P_CLS_LINK_ACCESS_g"      link_access_g      :: Ptr CInt
